/*
 * DelimitedSerialDevice.java
 *
 */

package sensor.delimitedSerial;

import sensor.PeriodicEventSource;
import sensor.PeriodicEventSourceImpl;
import util.BufferedReader;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.io.Reader;

public class DelimitedSerialDevice extends PeriodicEventSourceImpl {
    private static final int BUFFER_SIZE = 80;
    
    private InputStream inputStream;
    private InputStreamReader isr;
    private boolean stop;
    int position;
    ReadThread readThread;
    
    String lastSentence;
    
    /**
     * Creates a new instance of DelimitedSerialDevice
     */
    public DelimitedSerialDevice(InputStream is) {
        this.inputStream = is;
        isr = new InputStreamReader(is);
        readThread = new ReadThread();
        readThread.start();
    }
    
    class ReadThread extends Thread {
        public void run() {
            StringBuffer buffer = new StringBuffer(BUFFER_SIZE);
            int i;
            while (!stop) {
                try {
                    buffer.append((char)(isr.read()));
                    
                    //Keep buffer from growing too big
                    if (buffer.length() > BUFFER_SIZE-1) {
                        buffer = new StringBuffer(BUFFER_SIZE);
                        throw new IndexOutOfBoundsException("Buffer overflow - size: " + BUFFER_SIZE);
                    }
                    
                    //Todo - may overflow before delim is set
                    if (delimiter != null && buffer.toString().endsWith(delimiter)) {
                        buffer.setLength(buffer.length() - delimiter.length()); //Trim delimiters
                        
                        lastSentence = buffer.toString();
                        
                        if(pushEnabled) {
                            fireDelimitedSerialListenerNewDelimitedSerialEvent(new DelimitedSerialEvent(buffer.toString()));
                        }
                        
                        buffer = new StringBuffer(BUFFER_SIZE);
                    }
                } catch (Throwable t) {
//                    t.printStackTrace();
                    //Can happen when we yank the comm port from under readLine
                }
            }
        }
    }
    
    public void stop() {
        stopPolling();
        stop = true;
        readThread.interrupt();
        if (inputStream != null) {
            try {
                inputStream.close();
            } catch (IOException ex) {
//                ex.printStackTrace();
            }
        }
        if (isr != null) {
            //This caused it to hang on br.close when no dosimeter attached.
//            try {
//                br.close();
//            } catch (IOException ex) {
//                ex.printStackTrace();
//            }
        }
    }
    
    /**
     * Utility field holding list of DelimitedSerialListeners.
     */
    private transient java.util.Vector DelimitedSerialListenerList;
    
    /**
     * Registers DelimitedSerialListener to receive events.
     * @param listener The listener to register.
     */
    public synchronized void addDelimitedSerialListener(DelimitedSerialListener listener) {
        
        if (DelimitedSerialListenerList == null ) {
            DelimitedSerialListenerList = new java.util.Vector();
        }
        DelimitedSerialListenerList.addElement(listener);
    }
    
    /**
     * Removes DelimitedSerialListener from the list of listeners.
     * @param listener The listener to remove.
     */
    public synchronized void removeDelimitedSerialListener(DelimitedSerialListener listener) {
        
        if (DelimitedSerialListenerList != null ) {
            DelimitedSerialListenerList.removeElement(listener);
        }
    }
    
    /**
     * Notifies all registered listeners about the event.
     *
     * @param event The event to be fired
     */
    private synchronized void fireDelimitedSerialListenerNewDelimitedSerialEvent(DelimitedSerialEvent event) {
        for (int i = 0; i < DelimitedSerialListenerList.size(); i++) {
            ((DelimitedSerialListener)DelimitedSerialListenerList.elementAt(i)).newDelimitedSerialEvent(event);
        }
    }
    
    protected void fireNewEvent() {
        if (lastSentence != null) {
            fireDelimitedSerialListenerNewDelimitedSerialEvent(new DelimitedSerialEvent(lastSentence));
            lastSentence = null;
        }
    }
    
    /**
     * Holds value of property pushEnabled.
     */
    private boolean pushEnabled;
    
    /**
     * Getter for property pushEnabled.
     * @return Value of property pushEnabled.
     */
    public boolean isPushEnabled() {
        return this.pushEnabled;
    }
    
    /**
     * Setter for property pushEnabled.
     * @param pushEnabled New value of property pushEnabled.
     */
    public void setPushEnabled(boolean pushEnabled) {
        this.pushEnabled = pushEnabled;
    }

    /**
     * Holds value of property delimiter.
     */
    private String delimiter;

    /**
     * Getter for property delimiter.
     * @return Value of property delimiter.
     */
    public String getDelimiter() {
        return this.delimiter;
    }

    /**
     * Setter for property delimiter.
     * @param delimiter New value of property delimiter.
     */
    public void setDelimiter(String delimiter) {
        this.delimiter = delimiter;
    }
    
}
